/*
 * $LastChangedRevision: 903 $
 * $LastChangedDate:: 2016-03-10 14:44:32 +0100#$
 *
 * Copyright (c) 2013, Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 */
#include "uartStdio.h"
#include "bl.h"
#include "MMUConfig.h"

#include "TcpIp_lwIP_BeagleBoneBlack.h"
#include "LNet.h"
#include "VersionInfo.h"
#include "TableStruct.h"
#include "Services.h"
#include "BlockServicesX2C.h"

#include "X2CCode/X2C.h"
#include "Hardware.h"
#include "Main.h"

#define RX_BUF_SIZE (16*1024)
#define TX_BUF_SIZE (16*1024)

#define LNET_BUF_SIZE (255)

/**
 * IPv4 address (byte1.byte2.byte3.byte4)
 * Set all bytes to 0 for DHCP -> else static.
 */
#define IP_BYTE1	0
#define IP_BYTE2	0
#define IP_BYTE3	0
#define IP_BYTE4	0
/**
 * TCP port.
 */
#define TCP_PORT ((uint16)12666)
/**
 * Host name.
 */
#define HOSTNAME ((char*)"bbb-demo")


#define IP_ADDRESS (((uint32)(IP_BYTE1)<<24)|((uint32)(IP_BYTE2)<<16)|((uint32)(IP_BYTE3)<<8)|(uint32)IP_BYTE4)

static tTcpIp tcpip;
static uint8 rxBuffer[RX_BUF_SIZE];
static uint8 txBuffer[TX_BUF_SIZE];

static tLNet lnet;
static uint8 lnetBuffer[LNET_BUF_SIZE];

volatile tTableStruct TheTableStruct;
volatile tTableStruct* TableStruct = &TheTableStruct;

/* used by init version info */
const struct {
	uint8 date[11];
	uint8 time[8];
} compilationDate = { __DATE__, __TIME__ };

/* private prototypes */
static void initVersionInfo(volatile tTableStruct* tblStruct, uint16 appVersion);

int main(void)
{
	uint32 assignedIpAddr;

	MMUConfigAndEnable();

	ConsoleUtilsInit();
	ConsoleUtilsSetType(CONSOLE_UART);

    initTableStruct();
    assignedIpAddr = initTcpIp(&tcpip, IP_ADDRESS, TCP_PORT, HOSTNAME, rxBuffer, RX_BUF_SIZE, txBuffer, TX_BUF_SIZE);
    if (assignedIpAddr > 0)
    {
    	uint8 ipv4Bytes[4];

    	ipv4Bytes[0] = (uint8)(assignedIpAddr >> 24);
    	ipv4Bytes[1] = (uint8)(assignedIpAddr >> 16);
    	ipv4Bytes[2] = (uint8)(assignedIpAddr >> 8);
    	ipv4Bytes[3] = (uint8)(assignedIpAddr & 0xFF);

    	ConsoleUtilsPrintf("\n*** NETWORK CONFIG ***");
    	ConsoleUtilsPrintf("\nIP address: %d.%d.%d.%d\n", \
    			ipv4Bytes[3], ipv4Bytes[2], ipv4Bytes[1], ipv4Bytes[0]);
    	ConsoleUtilsPrintf("Hostname: %s\n", HOSTNAME);
    }
    initLNet(&lnet, lnetBuffer, LNET_BUF_SIZE, (uint8)1);
    initServiceTable((tProtocol*)&lnet);
    addCoreServices((tProtocol*)&lnet);
    addBlockServices((tProtocol*)&lnet);
    addTableStructProtocol((tProtocol*)&lnet);

    linkTcpIp((tProtocol*)&lnet, &tcpip);

    initVersionInfo(TableStruct, (uint16)0x0001);
    TableStruct->DSPState = PRG_LOADED_STATE;

    TableStruct->TFncTable = blockFunctionTable;
    TableStruct->TParamTable = parameterIdTable;

    X2C_Init();

    initHardware();

    while (1)
    {
		/* communication procedure */
		lnet.pCommunicate((tProtocol*)&lnet);
    }
}

/**
 * Adds frame compilation date, time and version info to X2C maintenance structure.
 */
static void initVersionInfo(volatile tTableStruct* tblStruct, uint16 appVersion)
{
	tblStruct->framePrgVersion = appVersion;
	tblStruct->framePrgCompDateTime = (uint8*)&compilationDate;
}

/* TODO call this task periodically */
void mainTask(void)
{
	X2C_Update();
	if (*(Outports.pLED2) > 0)
	{
		USER_LED2_ON;
	}
	else
	{
		USER_LED2_OFF;
	}
}
